;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------
;
; Entity Manager Header
;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Globals
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.globl man_entity_init                      ;/
.globl man_entity_getArray                  ;|
.globl man_entity_getArrayEnemies           ;|
.globl man_entity_getNumEnemies             ;|
.globl man_entity_create                    ;|
.globl man_entity_destroy                   ;|
.globl man_destroy_all_entities
.globl man_entity_forall_collision          ;| Functions
.globl man_entity_forall_matching           ;|
.globl man_entity_set4destruction           ;|
.globl man_entity_update                    ;|
.globl man_entity_freeSpace                 ;|
.globl man_entity_tilerow_entity_update     ;|
.globl man_entity_update_forall_position    ;\
.globl man_entity_forall_render
.globl man_entity_increment_prev_pos_y
.globl man_entity_limit_crossed
.globl create_player
.globl create_enemy
.globl create_bat
.globl create_blob
.globl create_frog
.globl create_turtle
.globl create_laser
.globl player_in_air

.globl e_type_invalid                       ;/
.globl e_type_render                        ;|
.globl e_type_movable                       ;|
.globl e_type_animated                      ;|
.globl e_type_input                         ;|
.globl e_type_ai                            ;|
.globl e_type_collisionable                 ;| Flags
.globl e_type_dead                          ;|
.globl e_type_default                       ;|
.globl e_type_ren_mov_inp_col               ;|
.globl e_type_ren_col                       ;|
.globl e_type_ren_mov                       ;|
.globl e_type_mov_ai                        ;|
.globl e_type_ren_mov_col_ai                ;\


.globl _spr_player                          ;/
.globl _spr_player_2
.globl _spr_enemy                           ;|
.globl _spr_bat                             ;| Sprites
.globl _spr_blob                            ;|
.globl _spr_frog                            ;\
.globl _spr_lasser
.globl _spr_viser

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Global constants
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SPR_PLAYER_W            = 6                     ;/
SPR_PLAYER_H            = 26                    ;| 
PLAYER_HALF_WEIGHT      = SPR_PLAYER_W / 2      ;| Player sprite constants
PLAYER_HALF_HEIGTH      = SPR_PLAYER_H / 2      ;|
MARGIN_ENTITY_PLAYER_X  = 1                     ;|
MARGIN_ENTITY_PLAYER_Y  = 2                     ;\

PLAYER_DEAD             = 0x00
PLAYER_LIFES            = 0x01
GRAVITY_PULL            = 1                     ; Acceleration applied to all gravitational entities
PLAYER_X_FORCE          = 4                     ; 
VPLAYER                 = 1                     ; Velocity of temporal player (TODO)
JUMP_FORCE              = -15                   ; Velocity which an enemy can jump, negative to indicate perpendicular force
Y_SCREEN_THRESHOLD      = 150

ENTITY_SIZE             = 24                    ;/ Entity constants
MAX_NUM_ENTITIES        = 20                    ;\   

UNDEFINED_COORD         = 0x00                  ;/ 
NO_PREV_POS             = 0x0000                ;| Null pointers
BEHAVIOURLESS           = 0x0000                ;|
NO_ANIMATION            = 0x0000                ;\
RENDER_FUNCTION         = 0x0000

SPR_ENEMY_W             = 4                     ;/ Enemy sprite constants
SPR_ENEMY_H             = 8                     ;\

SPR_VISER_W             = 3                     ;/ Viser sprite constants
SPR_VISER_H             = 6                    ;\

SPR_BLOB_W              = 8                     ;/ Blob sprite constants
SPR_BLOB_H              = 11                    ;\

SPR_FROG_W              = 8                     ;/ Frog sprite constants
SPR_FROG_H              = 16                    ;\

SPR_LASSER_W            = 32
SPR_LASSER_H            = 3

SPR_PATROL_W             = 3                     ;/ Patrol sprite constants
SPR_PATROL_H             = 6                    ;\

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Entity type flags
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
E_TYPE_INVALID          = 0x00                  ; Flags for invalid entities
E_TYPE_RENDER           = 0x01                  ; Flag for renderable entities
E_TYPE_MOVABLE          = 0x02                  ; Flag for movable entities
E_TYPE_PLAYER           = 0x04                  ; Flag for entities wich recieve input information
E_TYPE_AI               = 0x08                  ; Flag for entities controllables by AI
E_TYPE_GRAVITATE        = 0x10                  ; Flag for entities whit gravitation interactions
E_TYPE_COLLISIONABLE    = 0x20                  ; Flag for entities with collision
E_TYPE_ANIMATED         = 0x40                  ; Flag for entities in the air (TODO)
E_TYPE_DEAD             = 0x80                  ; Flag for dead entities
E_TYPE_DEFAULT          = 0x7F                  ; Flag for default entities

;; Adri
;E_TYPE_PLAYER = E_TYPE_RENDER|E_TYPE_MOVABLE|E_TYPE_PLAYER
E_TYPE_FLYING = E_TYPE_RENDER|E_TYPE_MOVABLE|E_TYPE_AI|E_TYPE_COLLISIONABLE|E_TYPE_ANIMATED
E_TYPE_FROG   = E_TYPE_RENDER|E_TYPE_MOVABLE|E_TYPE_AI|E_TYPE_GRAVITATE
E_TYPE_LASER  = E_TYPE_RENDER|E_TYPE_MOVABLE|E_TYPE_AI|E_TYPE_COLLISIONABLE|E_TYPE_ANIMATED

;; Carlos
E_TYPE_P = E_TYPE_RENDER|E_TYPE_MOVABLE|E_TYPE_PLAYER|E_TYPE_GRAVITATE | E_TYPE_ANIMATED
E_TYPE_REN_COL = E_TYPE_RENDER|E_TYPE_COLLISIONABLE


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Entity structure indexes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
TYPE_ENTITY_POS         = 0
X_ENTITY_POS            = 1
Y_ENTITY_POS            = 2
VX_ENTITY_POS           = 3
VY_ENTITY_POS           = 4
VX_MAX_POS              = 5
VY_MAX_POS              = 6
AX_ENTITY_POS           = 7
AY_ENTITY_POS           = 8
W_ENTITY_POS            = 9
H_ENTITY_POS            = 10
SPRT_ENTITY_POS_L       = 11
SPRT_ENTITY_POS_H       = 12
PREV_SPRT_ENTITY_POS_L  = 13
PREV_SPRT_ENTITY_POS_H  = 14
PREV_PTR_POS_L          = 15
PREV_PTR_POS_H          = 16
BEHAVIOUR_POS_L         = 17
BEHAVIOUR_POS_H         = 18
ANIMATION_POS_L         = 19
ANIMATION_POS_H         = 20
ANIM_STATUS             = 21
RENDER_FUNCTION_POS_L   = 22
RENDER_FUNCTION_POS_H   = 23


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Macros
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro DefineEntityAnnonimous  _type, _x, _y, _vx, _vy, _vxmax, _vymax, _ax, _ay, _w, _h, _sprite, _prev_spr, _prev_pos, _behaviour, _animation, _anim_status, RENDER_FUNCTION
   .db  _type, _x, _y, _vx, _vy, _vxmax, _vymax, _ax, _ay, _w, _h
   .dw  _sprite, _prev_spr, _prev_pos, _behaviour, _animation
   .db  _anim_status
   .dw  RENDER_FUNCTION
.endm

.macro DefineEntity  _name, _type, _x, _y, _vx, _vy, _vxmax, _vymax, _ax, _ay, _w, _h, _sprite, _prev_spr, _prev_pos, _behaviour, _animation, _anim_status, RENDER_FUNCTION
    _name::
        DefineEntityAnnonimous _type, _x, _y, _vx, _vy, _vxmax, _vymax, _ax, _ay, _w, _h, _sprite, _prev_spr, _prev_pos, _behaviour, _animation, _anim_status, RENDER_FUNCTION
.endm

.macro DefineComponentArrayStructureEntities _Tname, _N, _DefineTypeMacroDefault
    _Tname'_num::    .db 0
    _Tname'_pend::   .dw _Tname'_array
    _Tname'_array::
        .rept _N
            _DefineTypeMacroDefault E_TYPE_INVALID, 0xDE, 0xAD, 0xDE, 0xAD, 0xDE, 0xAD, 0xDE, 0xAD, 0xDE, 0xAD, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0, 0x0000
        .endm
    .db E_TYPE_INVALID
    _player_alive:: .db PLAYER_LIFES
.endm

