;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Includes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cpctelera.h.s"
.include "entity.h.s"
.include "sys/ai.h.s"
.include "sys/input.h.s"


.include "sys/collisions.h.s"
.include "sys/render.h.s"
.include "man/animations.h.s"

.include "cpcteleraHeader.h.s"
.include "game.h.s"
.include "platform.h.s"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Variables/Flags
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
max_entities            == 12
e_type_invalid          == 0x00     ; Flags for invalid entities
e_type_render           == 0x01     ; Flag for renderable entities
e_type_movable          == 0x02     ; Flag for movable entities
e_type_input            == 0x04     ; Flag for animated entities
e_type_ai               == 0x08     ; Flag for entities controllables by AI
e_type_animated         == 0x10     ; Flag for entities controllables by AI
e_type_collisionable    == 0x20     ; Flag for entities with collision
e_type_dead             == 0x80     ; Flag for dead entities
e_type_default          == 0x7F     ; Flag for default entities
e_type_ren_mov_inp_col  == e_type_render | e_type_movable | e_type_input | e_type_collisionable
e_type_ren_col          == e_type_render | e_type_collisionable
e_type_ren_mov          == e_type_render | e_type_movable
e_type_mov_ai           == e_type_movable | e_type_ai
e_type_ren_mov_col_ai   == e_type_render | e_type_movable | e_type_collisionable | e_type_ai

;; Variables definitions
DefineComponentArrayStructureEntities _entity, MAX_NUM_ENTITIES, DefineEntityAnnonimous
type: .db 0
player_in_air:: .db 0

;; PLAYER                 Name                 Type         X    Y   VX   VY  VXMAX  VYMAX  AX   AY         Width              Height            Sprite         Previous pos.                 Behaviour                   Animation
DefineEntity        _default_player,      E_TYPE_P,         20,  64,  0,   0,   1,     8,    0,   0,     SPR_PLAYER_W,      SPR_PLAYER_H,      _spr_player_2,         _spr_player_2,  NO_PREV_POS,                BEHAVIOURLESS                    ,_animation_player_falling, anim_status_player_falling, sys_render_first_one_entity
;; ENEMIES                Name                 Type         X    Y   VX   VY  VXMAX  VYMAX  AX   AY         Width              Height            Sprite         Previous pos.                 Behaviour                  , Animation, 0x0000
DefineEntity         _default_enemy,      E_TYPE_FLYING,   10,  150,  0,   0,   1,     1,    0,   0,     SPR_ENEMY_W,        SPR_ENEMY_H,      _spr_enemy,      _spr_enemy, NO_PREV_POS,                 BEHAVIOURLESS                ,NO_ANIMATION,0, sys_render_first_one_entity
DefineEntity         _default_viser,        E_TYPE_FLYING,   10,  20,   0,   0,   2,     3,    0,   0,      SPR_VISER_W,          SPR_VISER_H,        _spr_viser,       _spr_viser, NO_PREV_POS  ,       sys_ai_behaviour_wait_player      ,_animation_viser_not_detected, 0,sys_render_first_one_entity
DefineEntity         _default_blob,       E_TYPE_FLYING,   10,  150,  0,   0,   2,     6,    0,   0,     SPR_BLOB_W,         SPR_BLOB_H,        _spr_blob,      _spr_blob, NO_PREV_POS ,        sys_ai_behaviour_move_to_player      ,NO_ANIMATION,0, sys_render_first_one_entity
DefineEntity         _default_frog,       E_TYPE_FROG,     60,  100,  0,   0,   1,    10,    0,   0,     SPR_FROG_W,         SPR_FROG_H,        _spr_frog,      _spr_frog, NO_PREV_POS ,        sys_ai_behaviour_jump_to_player      ,NO_ANIMATION, 0,sys_render_first_one_entity
DefineEntity         _default_laser,      E_TYPE_LASER,     0,  #0xF0,  0,  0,   1,    #LASER_TOGGLE_CYCLES, 0, 0,  SPR_LASSER_W, SPR_LASSER_H,        _spr_lasser,       _spr_lasser, NO_PREV_POS,          sys_ai_behaviour_change_state, _animation_laser_on, anim_status_laser_on, sys_render_first_one_entity
DefineEntity         _default_patrol,     E_TYPE_FLYING,    0,    0,    0,  0,   0,   30,    0,   0,       SPR_PATROL_W,         SPR_PATROL_H,        _spr_patrol_1,      _spr_patrol_1, NO_PREV_POS,         sys_ai_behaviour_turtle      ,_animation_patrol,0, sys_render_first_one_entity


create_player::
    LD   HL, #_default_player
    CALL man_entity_create
    ld hl, #player_in_air
    ld (hl), #0x00
RET

create_enemy::
    LD   HL, #_default_enemy
    CALL man_entity_create
RET

create_bat::
    LD   HL, #_default_viser
    CALL man_entity_create
RET

create_blob::
    LD   HL, #_default_blob
    CALL man_entity_create
RET

create_frog::
    LD   HL, #_default_frog
    CALL man_entity_create
RET

; Input:
;   A
create_turtle::
    LD HL, #_default_patrol
    CALL man_entity_create
RET

create_laser::
    LD HL, #_default_laser
    CALL man_entity_create
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Sets the number of entities to 0 and the pointer to the last element 
;;  to the first element of the entity array and creates initial entities
;; INPUT: -
;; DESTROY: AF, HL
man_entity_init:
    XOR A                   ; Reset to 0 the a value
    LD (_entity_num), A     ; Sets _entity_num to 0

    LD HL, #_entity_array   ;/ _entity_pend = mem_pos(_entity_array)
    LD (_entity_pend), HL   ;\

    CALL create_player
    ;; Later move enemies to other funct
    ; CALL create_bat
    ; CALL create_bat

    ; CALL create_blob
    ; CALL create_frog
    ;CALL create_turtle
    ;LD Y_ENTITY_POS(IX), #150
    ; CALL create_turtle
RET


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Gets information relative to the array of entities
;; INPUT:
;; DESTROYS: AF, IX
;; RETURNS:
;;      IX - Entity array - memory adress
man_entity_getArray:
    LD IX, #_entity_array
RET


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Gets information relative to the array of entities + 1
;; INPUT:
;; DESTROYS: AF, BC, IX
;; RETURNS:
;;      IX - Entity array - memory adress
man_entity_getArrayEnemies:
    LD IX, #_entity_array
    ;; INC #ENTITY_SIZE(IX)
    LD BC, #ENTITY_SIZE        ; Estas dos líneas se pueden quitar?
    ADD IX, BC
RET


man_entity_getNumEnemies:
    LD  A, (_entity_num)
    DEC A
RET


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Adds a new entity component to the array without
;;  initializing it. It does not perform any check for space in the array.
;; DESTROY: F, HL, BC, DE
;; RETURN:
;;      DE - Points to added element
;;      BC - Sizeof (Entity)
man_entity_new::
    LD   HL, #_entity_num     ; /Increment number of reserved entities
    INC (HL)                  ; \

    LD    HL, (_entity_pend)  ;/
    LD     D, H               ;|
    LD     E, L               ;| HL = _entity_pend + sizeof_e = next free element in the array
    LD    BC, #ENTITY_SIZE    ;| _entity_pend = hl
    ADD   HL, BC              ;|
    LD    (_entity_pend), HL  ;\
RET


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Create a new entity by copying an entity into the 
;;  last element to add position(_last_element_ptr)
;;  INPUT:
;;      hl - Pointer to the entity to create
;;  DESTROY: HL, BC, DE, IX
man_entity_create::
    LD   (ptr_hl), HL
    CALL man_entity_new     ; Alocate space for a new entity

    ;; IX = DE              ; /
    ld__ixh_d               ; | Put new entity memory direction
    ld__ixl_e               ; \

    ld RENDER_FUNCTION_POS_L(ix), #sys_render_first_one_entity
    ld RENDER_FUNCTION_POS_H(ix), #1+sys_render_first_one_entity

    ptr_hl = .+1
    LD  HL, #0
    LDIR
RET


man_entity_limit_crossed::
    CALL man_entity_getArrayEnemies                ; Puede callearse directo
    _limit_loop:
        LD  A, TYPE_ENTITY_POS(IX)               ; Get entity type
        CP  #E_TYPE_INVALID
        RET Z                               ; Exit if invalid

        LD  A, Y_ENTITY_POS(IX)               ; Get entity type
        cp #0xE8
        jr c, #_limit_next_entity
        call man_entity_set4destruction

        _limit_next_entity:                             ; If not dead point to next entity
            LD   BC, #ENTITY_SIZE
            ADD  IX, BC
            JR   #_limit_loop

RET


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Update the entity_array by destroying the dead entities
;;  INPUT: - 
;;  DESTROY: IX, AF, BC, HL, DE
man_entity_update::
    CALL man_entity_getArrayEnemies                ; Puede callearse directo

    loop_ent_update:
    LD  A, TYPE_ENTITY_POS(IX)               ; Get entity type
    CP  #E_TYPE_INVALID
    RET Z                               ; Exit if invalid
    
    AND #E_TYPE_DEAD
    jr z, #next_e
    PUSH IX
    CALL man_entity_destroy
    POP IX
    JR   #loop_ent_update

    next_e:                             ; If not dead point to next entity
    LD   BC, #ENTITY_SIZE
    ADD  IX, BC
    JR   loop_ent_update


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Destroy a dead entity (TODO: Needs rework)
;;  INPUT:
;;      IX - MemPos of the entity to destroy
;;  DESTROY:
;;      IX, HL, DE, BC, A
man_entity_destroy::
  ld a, (_entity_num)
  dec a
  ld (_entity_num), a
;   cp #0x00

;   jr nz, #entity_destroy_no_single

;   ld ix, (_entity_pend)
;   ld TYPE_ENTITY_POS(ix), #E_TYPE_INVALID  ; Seteamos el ultimo elemento a INVALIDO
;   ld bc, #-ENTITY_SIZE
;   add ix, bc
;   ld (_entity_pend), ix ; Movemos el puntero al ultimo elemento una posicion a la izquierda

;   ld TYPE_ENTITY_POS(ix), #E_TYPE_INVALID  ; Seteamos el ultimo elemento a INVALIDO
;   ret   ; Si solo habia un elemento 
  
;   entity_destroy_no_single:

  ld hl, (_entity_pend)
  ld bc, #ENTITY_SIZE
  sbc hl, bc
  push ix
  pop de
  ldir ; Copiamos el ultimo elemento a la posicion de borrar

  ld ix, (_entity_pend)
  ld bc, #-ENTITY_SIZE
  add ix, bc
  ld TYPE_ENTITY_POS(ix), #E_TYPE_INVALID  ; Seteamos el ultimo elemento a INVALIDO
  ld (_entity_pend), ix ; Movemos el puntero al ultimo elemento una posicion a la izquierda
RET


; Destruye todas las entidades del
; array de entidades.
; Modifica:
;   AF, BC, IX
man_destroy_all_entities::
    ld ix, #_entity_array
    ld a, (_entity_num)
    cp #0x00
    ret z

    _loop_destroy_all:
    push af
    ld a, #0x00
    ld (ix), a      ; Asumo que el tipo esta en la primera posicion

    pop af
    dec a
    cp #0x00
    jr z, #_destroy_all_fin ; Comprobamos si hemos borrado todos los elementos del array


    ld bc, #ENTITY_SIZE
    add ix, bc              ; Iteramos al siguiente elemento

    jr #_loop_destroy_all 

    _destroy_all_fin:
    ld a, #0x00
    ld (_entity_num), a

    ld hl, #_entity_array
    ld (_entity_pend), hl
RET



; Carlos
man_entity_forall_collision::
    ;ld (ptr_entity_de), ix
    call man_entity_getArray
    loop_entity_forall_collision:
        ld a, TYPE_ENTITY_POS(ix)
        cp #e_type_invalid
        ret z
        ld hl, #sys_collision_update_one_entity
        push hl
        ld de, #pop_entity_forall_collision 
        push de
        jp (hl)

        pop_entity_forall_collision: pop hl
        ld bc, #ENTITY_SIZE
        add ix, bc
        
        jr loop_entity_forall_collision


; Incrementa en una posicion la prev_y
; de la entidad pasada por parametro
; Input:
;   IX -> Puntero a la entidad a modificar
man_entity_increment_prev_pos_y::
  inc PREV_PTR_POS_H(ix)
  ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Apply a function to all non invalid entities
;;  INPUT:
;;      HL - MemPos of the function to apply
;;      A - e_type
;;  DESTROY:
;;      IX, HL, DE, BC, A
man_entity_forall_matching::
    call man_entity_getArray
    ld (type), a
    loop_entity_forall_matching:
        ld a, (type)
        ld c, a
        ld a, TYPE_ENTITY_POS(ix)
        cp #E_TYPE_INVALID
        ret z
        and c
        cp c
        jr nz, loop_next_entity
        push hl
        ld de, #pop_entity_forall_matching 
        push de
        jp (hl)

        pop_entity_forall_matching: pop hl
        loop_next_entity:
            push bc
            ld bc, #ENTITY_SIZE
            add ix, bc
            pop bc
            ld a, c
        jr loop_entity_forall_matching

; Adri       ;; Double check to latter optimizations
; man_entity_forall_matching:
;     LD IX, #_entity_array
;     _e_matching_loop:
;         LD   A, TYPE_ENTITY_POS(IX)
;         CP   #E_TYPE_INVALID
;         RET  Z
        
;         ;; Store for avoiding modifications
;         PUSH BC

;         AND  B
;         CP   B
;         JR  NZ, avoid_function_matching
;             PUSH HL

;             ;; IoC
;             LD   BC, #return_point_matching
;             PUSH BC
;             JP   (HL)
;             return_point_matching:
            
;             POP HL
;         avoid_function_matching:

;         ;; Obtain next entity on array
;         LD   BC, #ENTITY_SIZE
;         ADD  IX, BC
        
;         ;; Retrieve for avoiding modifications
;         POP BC

;     JR _e_matching_loop



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Apply a function to all non invalid entities
;;  INPUT:
;;  DESTROY:
;;      IX, HL, DE, BC, A
man_entity_forall_render::
    call man_entity_getArray

;    cpctm_setBorder_asm HW_BLUE
    loop_entity_forall_render:
        ld a, TYPE_ENTITY_POS(ix)
        cp #E_TYPE_INVALID
        ret z

        ld de, #pop_entity_forall_render 
        push de

        ld h, RENDER_FUNCTION_POS_H(ix)
        ld l, RENDER_FUNCTION_POS_L(ix)
        jp (hl)

        pop_entity_forall_render: 

        ld bc, #ENTITY_SIZE
        add ix, bc
        jr loop_entity_forall_render



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Set an entity for destruction by putting it type 
;;  to e_type_dead
;;  INPUT:
;;      IX - MemPos of the entity to set for destruction
;;  DESTROY:
;;      A
man_entity_set4destruction:
    LD A, #E_TYPE_DEAD
    OR TYPE_ENTITY_POS(IX)
    LD TYPE_ENTITY_POS(IX), A
RET


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Set the register A the value of the free space in the entity array
;;  OUTPUT:
;;      A
;;  DESTROY:
;;      A, BC
man_entity_freeSpace::
    LD   A, #MAX_NUM_ENTITIES
    LD  BC, (_entity_num)
    SUB  C
RET

; Genera las entidade de una fila
; del tilemap posicinadas en la
; parte inferior de la pantalla.
; Input:
;   IX -> Fila de datos de entidades
; Modifica:
;   HL
man_entity_tilerow_entity_update::
  .macro _mult_a_4
    add a
    add a
  .endm

  _elementos_restantes = .+1
  ld a, #0x10

  _tilerow_entity_update_loop:
  push ix
  ld a, (ix)
  cp #0x01                  ; Si es entidad -> La creo
  jr z, #_generate_type_1


  _tilerow_entity_update_continue: 
  pop ix
  inc ix
  ld a, (_elementos_restantes)
  dec a
  jr z, #_fin_reset

  ld (_elementos_restantes), a
  jr #_tilerow_entity_update_loop

  _generate_type_1:
  ld hl, #_default_platform
  call man_platform_create

  
  ld a, (_elementos_restantes)
  ld b, a
  ld a, #16
  sub b
  _mult_a_4

  ld p_x(ix), a ; Almacenamos la posicion X de la plataforma

  ld a, #0x10
  ld p_y(ix), a ; Almacenamos la posicion Y de la plataforma


  jr #_tilerow_entity_update_continue


  _fin_reset:
  ld hl, #_elementos_restantes
  ld (hl), #0x10

  ret


; Mueve la posicion de todas las entidades
; en el array de entidades (no aplica al
; protagonista)
; Modifica:
;   AF, BC, IX
man_entity_update_forall_position::
  call man_entity_getArrayEnemies

  _update_forall_position_loop:
    ld a, TYPE_ENTITY_POS(ix)
    cp #E_TYPE_INVALID
    ret z ; Si es de tipo invalido, salimos del bucle

    ld a, Y_ENTITY_POS(ix)
    sub #NUM_POSICIONES_SCROLL
    cp #0xF0
    JP nz, #_update_forall_position_NO_marcar_destruir

    call man_entity_set4destruction     ; La marcamos para destruir si (p_y - #0xF8 == 0)
    ret

    _update_forall_position_NO_marcar_destruir:
    ld Y_ENTITY_POS(ix), a
    ld bc, #ENTITY_SIZE
    add ix, bc      ; Iteramos al siguiente elemento
    jr #_update_forall_position_loop

RET
