;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Includes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "game.h.s"
.include "entity.h.s"
.include "platform.h.s"
.include "sys/ai.h.s"
.include "sys/input.h.s"
.include "sys/collisions.h.s"
.include "sys/physics.h.s"
.include "sys/render.h.s"
.include "sys/animations.h.s"
.include "tilemap.h.s"
.include "cpcteleraHeader.h.s"
.include "level.h.s"
.include "cpctelera.h.s"
.include "music.h.s"

man_game_restart_level:

    LD   A, #DIE_SFX
    CALL man_music_activate_sfx


    call sys_render_displaySetUp

    call man_destroy_all_entities   ; Destruimos las entidades
    call man_destroy_all_platforms  ; Destruimos las plataformas

    call man_level_set_new_map      ; Recargamos el nivel actual
    call man_entity_init

;    call action_move_up
    ld a, #PLAYER_LIFES
    ld (_player_alive), a
    ld a, #0
    ld (player_in_air), a

    ret

man_game_over::
    call man_entity_getArray
    ld a, TYPE_ENTITY_POS(ix)
    cp #E_TYPE_INVALID
    ret z                           ; Si la entidad es invalida -> Ret

    ld a, (_player_alive)
    cp #PLAYER_DEAD
    ret nz                          ; Si la entidad no esta muerta -> Ret

    call man_game_restart_level
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sets initial values used in game before starting a new game.
;; INPUT:
;; DESTROYS:
man_game_init:
    CALL man_music_init
    CALL man_level_init
    CALL man_entity_init
    CALL man_platform_init

    CALL man_entity_getArray
    CALL sys_ai_init
    CALL sys_physics_init
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sets initial values used in game before starting a new game.
;; INPUT:
;; DESTROYS:
man_game_update:
    CALL sys_ai_update
    CALL sys_input_update
    CALL sys_physics_update
    CALL man_entity_update              
    CALL sys_collision_update
    CALL sys_animations_update
    call man_entity_limit_crossed
    call man_platform_update
    call man_game_over
    call man_game_restart
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Performs rendering process apart from game update to account 
;; for its time constraints
;; INPUT:
;; DESTROYS:
man_game_render:
    CALL cpct_waitVSYNCStart_asm
    CALL sys_render_update
RET


; Delvuelve A a 1 si se tiene que reiniciar
man_game_restart::
    ld a, (actual_action)
    cp #action_restart
    ret nz
    call man_game_restart_level
RET

