;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------
;
; Level Manager
;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Includes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "level.h.s"
.include "tilemap.h.s"
.include "../sys/render.h.s"
.include "game.h.s"
.include "platform.h.s"
.include "entity.h.s"
.include "cpctelera.h.s"

.include "music.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Funciones
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; Inicializa el mapa y los tiles
man_level_init::
  call man_level_set_new_map
RET
  ld hl, (level_list_maps)
;  ld (current_level_map), hl
;  ld e, (hl)
;  inc hl
;  ld d, (hl)
;  ex de, hl
  call sys_tilemap_set_current_map_line_mem

  ld hl, (level_list_tiles)
;  ld (current_level_tile), hl
;  ld e, (hl)
;  inc hl
;  ld d, (hl)
;  ex de, hl
  call sys_tilemap_set_current_tile_mem
RET

; Set current_level_map
; Input:
;   HL
man_level_set_current_map::
  ld (current_level_map), hl
RET

; Get current_level_tile
; Output:
;   HL
man_level_get_current_map::
  ld hl, (current_level_map)
RET

; Set current_level_tile
; Input:
;   HL
man_level_set_current_tile::
  ld (current_level_tile), hl
RET

; Get current_level_tile
; Output:
;   HL
man_level_get_current_tile::
  ld hl, (current_level_tile)
RET

; Set chunks_restantes
; Input:
;   HL
man_level_set_chunks_restantes::
  ld (chunks_restantes), hl
RET

; Get chunks_restantes
; Output:
;   HL
man_level_get_chunks_restantes::
  ld hl, (chunks_restantes)
RET


; Decrementa en una unidad los chunks
; restantes
; Modifica:
;   HL, BC
man_level_decrementar_chunks_restantes::
  ld hl, (chunks_restantes)
  ld bc, #0x01
  add hl, bc
  ld (chunks_restantes), hl
RET


; Actualiza el current level map
; al siguiente mapa en level_list_maps
; Modifica;
;   AF, HL, BC
man_level_load_next_level::
  ld a, (level_index_maps_tiles)
  ld c, #0x02
  add c
  ld (level_index_maps_tiles), a      ; Actualizamos el indice actual

  ld b, #0x00
  ld c, a

  CALL man_music_change_track
  LD   A, #COMPLETED_SFX
  CALL man_music_activate_sfx

  ld hl, #level_list_maps
  add hl, bc
  ld (current_level_map), hl          ; Actualizamos el puntero al mapa actual

  ld hl, #level_list_tiles
  add hl, bc
  ld (current_level_tile), hl         ; Actualizamos el puntero al tile actual

  ld hl, #level_list_tams
  add hl, bc
  ld (current_level_tam), hl          ; Actualizamos el puntero al tam del mapa actual
RET


; Setea el map y las tiles para render
; Modifia:
;   HL
man_level_set_new_map::

  ld hl, (current_level_map)
  ld e, (hl)
  inc hl
  ld d, (hl)
  ex de, hl
  call sys_tilemap_set_current_map_line_mem

  ld hl, (current_level_tile)
  ld e, (hl)
  inc hl
  ld d, (hl)
  ex de, hl
  call sys_tilemap_set_current_tile_mem


  ld hl, (current_level_tam)
  ld a, (hl)
  ld (chunks_restantes), a


  ld a, (level_index_maps_tiles)
  cp #2
  jr z, change_palette_2

  ld a, (level_index_maps_tiles)
  cp #10
  jr z, change_palette_3

  ld a, (level_index_maps_tiles)
  cp #12
  jr z, change_palette_2

  ld a, (level_index_maps_tiles)
  cp #16
  jr z, change_palette_4

  ld a, (level_index_maps_tiles)
  cp #22
  jr z, change_palette_5
      LD   HL, #_g_palette
      jr set_palette
  change_palette_2:
    LD   HL, #_g_palette_level_2
    jr set_palette
  change_palette_3:
    LD   HL, #_g_palette_level_3
    jr set_palette

  change_palette_4:
    LD   HL, #_g_palette_level_4
    jr set_palette

  change_palette_5:
    LD   HL, #_g_palette_level_5

  set_palette:
  LD   DE, #16
  CALL cpct_setPalette_asm
  ret
  
RET

; Decrementa una unidad los chunks
; del mapa restantes
; Modifica:
;   AF, HL
man_level_dec_chunks_restantes::
  ld a, (chunks_restantes)
  dec a
  ld (chunks_restantes), a
  cp #0x00
  ret nz

  ;cpctm_clearScreen_asm #0x00
  call man_destroy_all_platforms
  ;call man_destroy_all_entities

  call man_level_load_next_level
  call man_level_set_new_map
RET


