;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------
.include "cpctelera.h.s"
.include "music.h.s"
.include "utils.h.s"

_sfx_status:        .db 0x00
_sfx_effect:        .dw 0x0000
_music_loaded:      .db 0x01                ; Play initialy music
_music_changed:     .db 0x00
_music_track:       .dw 0x0000
_music_fade_in:     .db 0x00
_music_fade_out:    .db 0x00
_music_volume:      .db 0x00

;; SFX Params
;; A = No Channel (0,1,2)
;; L = Instrument Number (>0)
;; H = Volume (0...F)
;; E = Note (0...143)
;; D = Speed (0 = As original, 1...255 = new Speed (1 is the fastest))
;; BC = Inverted Pitch (-#FFFF -> FFFF). 0 is no pitch. The higher the pitch, the lower the sound.

;; A
man_music_set_dying_sfx:
    DEC A
    LD  (_sfx_status), A
    JR  Z, _second_dying_step
        LD   A, #0x02                    ; Primer canal
        LD   L, #1                      ; Hit instrumento
        LD   H, #0x0F
        LD   E, #42                     ; No se que nota es
        LD   D, #0                      ; Original
        LD  BC, #0
        JR  exit_dying_sfx
    _second_dying_step:
        LD   A, #0x02                   ; Primer canal
        LD   L, #1                      ; Hit instrumento
        LD   H, #0x0F
        LD   E, #20                     ; No se que nota es
        LD   D, #0                      ; Original
        LD  BC, #0
    exit_dying_sfx:
RET

man_music_set_jump_sfx:
    DEC A
    LD  (_sfx_status), A
    JR  Z, _second_jump_step
        LD   A, #0x02                   ; Segundo canal
        LD   L, #3                      ; Hit instrumento
        LD   H, #0x0F
        LD   E, #42                      ; No se que nota es
        LD   D, #0                      ; Original
        LD  BC, #0
        JR  exit_jump_sfx
    _second_jump_step:
        LD   A, #0x02                   ; Segundo canal
        LD   L, #3                      ; Hit instrumento
        LD   H, #0x0F
        LD   E, #44                     ; No se que nota es
        LD   D, #0                      ; Original
        LD  BC, #0
    exit_jump_sfx:
RET

man_music_set_completed_sfx:
    DEC A
    LD  (_sfx_status), A
    JR  Z, _second_completed_step
        LD   A, #0x02                   ; Segundo canal
        LD   L, #3                      ; Hit instrumento
        LD   H, #0x0F
        LD   E, #66                     ; No se que nota es
        LD   D, #0                      ; Original
        LD  BC, #0
        JR  exit_completed_sfx
    _second_completed_step:
    DEC A
    JR  Z, _third_completed_step
        LD   A, #0x02                   ; Segundo canal
        LD   L, #3                      ; Hit instrumento
        LD   H, #0x0F
        LD   E, #68                     ; No se que nota es
        LD   D, #0                      ; Original
        LD  BC, #0
        JR  exit_completed_sfx
    _third_completed_step:
        LD   A, #0x02                   ; Segundo canal
        LD   L, #3                      ; Hit instrumento
        LD   H, #0x0F
        LD   E, #73                     ; No se que nota es
        LD   D, #0                      ; Original
        LD  BC, #0
    exit_completed_sfx:
RET

;; A
man_music_activate_sfx:
    CP #DIE_SFX
    JR NZ, _skip_die_sfx
       LD  A, #SFX_DIE_LEN
       LD  (_sfx_status), A
       LD  HL, #man_music_set_dying_sfx
       LD  (_sfx_effect), HL
       JR  _exit_activation
    _skip_die_sfx:

    CP #JUMP_SFX
    JR NZ, _skip_jump_sfx
        LD  A, #SFX_JUMP_LEN
        LD  (_sfx_status), A
        LD  HL, #man_music_set_jump_sfx
        LD  (_sfx_effect), HL
    _skip_jump_sfx:

    CP #COMPLETED_SFX
    JR NZ, _skip_completed_sfx
        LD  A, #SFX_COMPLETED_LEN
        LD  (_sfx_status), A
        LD  HL, #man_music_set_completed_sfx
        LD  (_sfx_effect), HL
    _skip_completed_sfx:

    _exit_activation:
RET

man_music_toggle:
    LD  A, (_music_loaded)
    XOR #1
    LD  (_music_loaded), A
    CALL cpct_akp_stop_asm
RET

man_music_init:
    LD   DE, #_sfx              ;; DE = Pointer to SFX Music
    CALL cpct_akp_SFXInit_asm   ;; Initialize instruments for SFX

    LD   DE, #_wormhole
    CALL cpct_akp_musicInit_asm
RET

man_music_change_track:
    CP #MENU_TRACK
    JR NZ, _skip_menu_track
       LD   DE, #_wormhole
       JR _exit_music_change
    _skip_menu_track:

    CP #LEVEL0_TRACK
    JR NZ, _skip_level0_track
        LD   DE, #_m_level_0
        JR _exit_music_change
    _skip_level0_track:

    CP #LEVEL4_TRACK
    JR NZ, _skip_level1_track
        LD   DE, #_m_level_1
        JR _exit_music_change
    _skip_level1_track:

    CP #LEVEL7_TRACK
    JR NZ, _skip_level2_track
        LD   DE, #_m_level_2
        JR _exit_music_change
    _skip_level2_track:

    CP #LEVEL10_TRACK
    JR NZ, _skip_level3_track
        LD   DE, #_m_level_3
        JR _exit_music_change
    _skip_level3_track:

    CP #FINAL_TRACK
    JR NZ, _skip_level4_track
        LD   DE, #_wormhole
        JR _exit_music_change
    _skip_level4_track:

    RET                         ;; If no track selected

    _exit_music_change:

    LD   A, #MUSIC_TRANSITION_CYCLES
    LD   (_music_changed), A
    LD   (_music_track), DE
RET

man_music_update:
    LD   A, (_music_changed)
    CP  #0
    JR   Z, _skip_change_track
        DEC  A
        LD   (_music_changed), A
        JR   NZ, _fade_out_
            INC  A
            LD   (_music_fade_in), A
            LD   DE, (_music_track)
            CALL cpct_akp_musicInit_asm
            JR   _skip_change_track
        _fade_out_:

        ;; Fade out
        LD   A, (_music_volume)
        INC  A
        LD   E, #FADE_UNDER_LIMIT
        CALL limit_value
        LD   (_music_volume), A
        LD   E, #FADE_DEVIATION_APPLIED
        CALL shift_value
        CALL cpct_akp_setFadeVolume_asm
    _skip_change_track:

    LD   A, (_sfx_status)
    LD  HL, (_sfx_effect)
    CP  #0
    JR   Z, _skip_sfx
        LD   BC, #_loaded_sfx
        PUSH BC
        JP   (HL)                     ;; Call SFX loader
        _loaded_sfx:

        CALL cpct_akp_SFXPlay_asm
    _skip_sfx:
    
    LD   A, (_music_loaded)
    DEC  A
    JR   NZ, _skip_music
        CALL cpct_akp_musicPlay_asm

        LD   A, (_music_fade_in)
        DEC  A
        JR  NZ, _skip_music
            LD   A, (_music_volume)
            DEC  A
            LD   (_music_volume), A
            JR   NZ, _fade_in_
                LD (_music_fade_in), A
            _fade_in_:
            LD   E, #FADE_DEVIATION_APPLIED
            CALL shift_value
            CALL cpct_akp_setFadeVolume_asm
    _skip_music:
RET
