;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------
;
; Platform Manager
;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Includes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "platform.h.s"
.include "sys/collisions.h.s"
.include "cpcteleraHeader.h.s"
.include "cpctelera.h.s"
.include "game.h.s"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Variables/Flags
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
max_platforms            == 200
E_TYPE_INVALID           == 0x00     ; Flags for invalid entities
E_TYPE_PLATAFORMA        == 0x01     ; Flag for renderable entities
E_TYPE_PINCHO            == 0x02     ; Flag para plataforma de tipo pincho
E_TYPE_DEAD              == 0x80     ; Flag for dead entities

DefineComponentArrayStructurePlatforms _platform, max_platforms, DefinePlatformAnnonimous
_m_zero_type_at_the_end: .db #E_TYPE_INVALID
DefinePlatformPointer _platform_pointers, max_platforms
type: .db 0

; Temporal, make dynamic                        ; Adri
DefinePlatform _default_platform, E_TYPE_PLATAFORMA, 10, 150, SPR_PLATFORM_W, SPR_PLATFORM_H

DefinePlatform _default_pincho, E_TYPE_PINCHO, 10, 150, SPR_PLATFORM_W, SPR_PLATFORM_H

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Set the register IX with the position of platform_array
;;  INPUT: -
;;  DESTROY:
;;      IX
platformman_getPlatformArray_IX::
    ld ix, #_platform_array
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Apply a function to all non invalid entities
;;  INPUT: -
;;  DESTROY:
;;      IY
platformman_getPlatformArray_IY::
    ld iy, #_platform_array
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Sets the number of entities to 0 and the pointer to the last element 
;;  to the first element of the platform array
;; INPUT: -
;; DESTROY: AF, HL
;;    
man_platform_init::
    ld a, #0x00
    ld (_platform_num), a

    ld hl, #_platform_array
    ld (_platform_pend), hl
    ;; Later move this definition to other function, just for testing
    ;ld hl, #_default_platform                ; Adri
    ;call man_platform_create
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Adds a new platform component to the array without
;;  initializing it. It does not perform any check for space in the array.
;; 
;; DESTROY: F, HL, BC, DE
;; RETURN:
;;      DE: Points to added element
;;      BC: Sizeof (platform)
man_platform_new::
    ;; Increment number of reserved entities
    ld hl, #_platform_num
    inc (hl)

    ;; Increment Array end pointer to point to the next free element in the array
    ld hl, (_platform_pend)
    ld d, h
    ld e, l
    ld bc, #sizeof_p
    add hl, bc
    ld (_platform_pend), hl

    ;ld hl, (_platform_pointers_pend)       ; No lo pongo porque creo que no lo necesito                   
    ;inc hl                                 ; No lo pongo porque creo que no lo necesito
    ;inc hl                                 ; No lo pongo porque creo que no lo necesito
    ;ld (_platform_pointers_pend), hl       ; No lo pongo porque creo que no lo necesito                            

    ret

man_platform_pointer_add::
    ld hl, #_platform_pointers_array
    ld e, (hl)
    inc hl
    ld d, (hl)
    inc hl
    ld a, e
    or d
    jr z, primer_elemento
    ld hl, #_platform_pointers_array
    recorrido:
        ld e, (hl)
        inc hl
        ld d, (hl)
        inc hl
        ld__iyl_e
        ld__iyh_d

        ld a, e
        or d
        jr z, ultimo_cambio
        
        ld a, p_y(iy)
        cp p_y(ix)
        jr nc, no_cambiar
        cambiar:
            dec hl
            dec hl
            ld__e_ixl
            ld__d_ixh
            ld (hl), e
            inc hl
            ld (hl), d
            inc hl
            ld__ix_iy

        no_cambiar:        
        jr recorrido

    primer_elemento:
        dec hl
        dec hl
        ld__e_ixl
        ld__d_ixh
        ld (hl), e
        inc hl
        ld (hl), d
        ret
    ultimo_cambio:
        dec hl
        dec hl
        ld__e_ixl
        ld__d_ixh
        ld (hl), e
        inc hl
        ld (hl), d
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Create a new platform by copying an platform into the 
;;  last element to add position(_last_element_ptr)
;;  INPUT:
;;      HL - Pointer to the platform to create
;;  DESTROY:
;;      DE, BC, IX, HL
man_platform_create::
    ld (ptr_hl), hl
    call man_platform_new

    ;; IX = DE
    ld__ixh_d
    ld__ixl_e
    ptr_hl = .+1
    ld hl, #0
    ;pop hl  ;; Recover hl
    ldir

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Update the platform_array by destroying the dead entities
;;  INPUT: -
;;  DESTROY:
;;      IX, HL, DE, BC, AF
man_platform_update::
    call platformman_getPlatformArray_IX
    loop_ent_update:
        ld a, p_type(ix)
        cp #E_TYPE_INVALID
        ret z
        and #E_TYPE_DEAD
        jr z, next_e
            push ix
            call man_platform_destroy
            pop ix
            jr loop_ent_update
        next_e:
            ld bc, #sizeof_p
            add ix, bc
            jr loop_ent_update

; Actualiza el estado de las plataformas
; que cruzan el limite superior de la pantalla.
; Modifica
;   IX, AF, BC
man_platform_crossed_limit_update::
  call platformman_getPlatformArray_IX

  _crossed_limit_update_loop:
    ld a, p_type(ix)
    cp #E_TYPE_INVALID
    ret z ; Si es de tipo invalido, salimos del bucle

    ld a, p_y(ix)
    cp #POSICION_SUPERIOR_PANTALLA    ; No queremos borrarla en 0x00, la queremos borrar cuando lo transpasa
    jr nz, #_crossed_limit_update_NO_marcar_destruir ; Si se pasa la posicion 0x00 -> Marcamos para destruir

    call man_platform_set4destruction     ; La marcamos para destruir si (p_y - #0xF8 == 0)

    _crossed_limit_update_NO_marcar_destruir:
    ld bc, #sizeof_p
    add ix, bc      ; Iteramos al siguiente elemento
    jr #_crossed_limit_update_loop

RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Apply a function to all non invalid entities
;;  INPUT:
;;      HL - MemPos of the function to apply
;;      IX - Puntero a la entidad 
;;  DESTROY:
;;      IX, HL, DE, BC, A
man_platform_forall_collision::
    ;ld (ptr_entity_de), ix

    ld iy, #_platform_array
    loop_platform_forall:
        ld a, p_type(iy)
        cp #E_TYPE_INVALID
        ret z
        cp #E_TYPE_PINCHO
        jr nz, collision_platform 
        ld hl, #sys_collision_update_one_pincho
        jr continue_forall_collision
        collision_platform:
            ld hl, #sys_collision_update_one_platform
        
        continue_forall_collision:
        push hl
        ld de, #pop_platform_forall 
        push de

        jp (hl)

        pop_platform_forall: pop hl
        ld bc, #sizeof_p
        add iy, bc
        
        jr loop_platform_forall

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Apply a function to all non invalid entities
;;  INPUT:
;;      HL - MemPos of the function to apply
;;      A - e_type
;;  DESTROY:
;;      IX, HL, DE, BC, A
man_platform_forall_matching::
    call platformman_getPlatformArray_IX
    ld (type), a
    loop_platform_forall_matching:
        ld a, (type)
        ld c, a
        ld a, p_type(ix)
        cp #E_TYPE_INVALID
        ret z
        and c
        cp c
        jr nz, loop_next_platform
        push hl
        ld de, #pop_platform_forall_matching 
        push de
        jp (hl)

        pop_platform_forall_matching: pop hl
        loop_next_platform:
            push bc
            ld bc, #sizeof_p
            add ix, bc
            pop bc
            ld a, c
        jr loop_platform_forall_matching


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Apply a funtion to all non invalid platforms
;;  matching pos_y = B
;;  INPUT:
;;      HL - MemPos of the function to apply
;;      A - Position Y
;;  DESTROY:
;;      IX, HL, DE, BC, A
man_platform_forall_matching_pos_y::
RET
    call platformman_getPlatformArray_IX

    _man_platform_forall_matching_pos_y_loop:
        ld c, a ; Salvamos la A

        ld a, p_type(ix)
        cp #E_TYPE_INVALID
        ret z           ; Comprobamos que no es invalida

        ld a, c ; Recuperamos la A
        ld b, p_y(ix)
        cp b    ; Comprobamos si la posicion de la plataforma es igual a A

        jr nz, #_man_platform_forall_matching_pos_y_next_platform ; Si no es la posicion Y ->  Continuamos

        push hl
        ld de, #_man_platform_forall_matching_pos_y_matching 
        push de
        jp (hl) ; Si es la posicion Y -> Inversion de control -> Funcion HL

        _man_platform_forall_matching_pos_y_matching: pop hl
        _man_platform_forall_matching_pos_y_next_platform:
            push bc
            ld bc, #sizeof_p
            add ix, bc
            pop bc
            ld a, c
        jr #_man_platform_forall_matching_pos_y_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Destroy a dead platform (TODO: Para mi futuro yo: Si lees esto, es posible que esto no vaya muy bien ps: descomentar código)
;;  IMORTANTE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! no borrar el código comentado ;(
;;  INPUT:
;;      IX - MemPos of the platform to destroy
;;  DESTROY:
;;      IX, HL, DE, BC, A
man_platform_destroy::
  ld a, (_platform_num)
  dec a
  ld (_platform_num), a
  cp #0x00

  jr nz, #platform_destroy_no_single

  ld ix, (_platform_pend)
  ld p_type(ix), #E_TYPE_INVALID  ; Seteamos el ultimo elemento a INVALIDO
  ld bc, #-sizeof_p
  add ix, bc
  ld (_platform_pend), ix ; Movemos el puntero al ultimo elemento una posicion a la izquierda

  ld p_type(ix), #E_TYPE_INVALID  ; Seteamos el ultimo elemento a INVALIDO
  ret   ; Si solo habia un elemento 
  
  platform_destroy_no_single:

  ld hl, (_platform_pend)
  ld bc, #sizeof_p
  sbc hl, bc
  ld__de_ix
  ldir ; Copiamos el ultimo elemento a la posicion de borrar
    
  ld ix, (_platform_pend)
  ld bc, #-sizeof_p
  add ix, bc
  ld p_type(ix), #E_TYPE_INVALID  ; Seteamos el ultimo elemento a INVALIDO
  ld (_platform_pend), ix ; Movemos el puntero al ultimo elemento una posicion a la izquierda
RET


; Destruye todas las plataformas del
; array de plataformas.
; Modifica:
;   AF, BC, IX
man_destroy_all_platforms::
    ld ix, #_platform_array
    ld a, (_platform_num)
    cp #0x00
    ret z

    _loop_destroy_all:
    push af
    ld a, #0x00
    ld p_type(ix), a      ; Asumo que el tipo esta en la primera posicion

    pop af
    dec a
    cp #0x00
    jr z, #_destroy_all_fin ; Comprobamos si hemos borrado todos los elementos del array


    ld bc, #sizeof_p
    add ix, bc              ; Iteramos al siguiente elemento

    jr #_loop_destroy_all 

    _destroy_all_fin:
    ld a, #0x00
    ld (_platform_num), a

    ld hl, #_platform_array
    ld (_platform_pend), hl
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Set an platform for destruction by putting it type 
;;  to E_TYPE_DEAD
;;
;;  INPUT:
;;      IX - MemPos of the platform to set for destruction
;;  DESTROY:
;;      A
man_platform_set4destruction::
    ld a, p_type(ix)
    or #E_TYPE_DEAD
    ld p_type(ix), a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Set the register A the value of the free space in the platform array
;;  OUTPUT:
;;      A
;;  DESTROY:
;;      A, BC
man_platform_freeSpace::
    ld a, #max_platforms
    ld bc, (_platform_num)
    sub c
    ret


; Genera las plataformas de una fila
; del tilemap posicinadas en la
; parte inferior del protagonista.
; Input:
;   IX -> Fila de datos de entidades
; Modifica:
;   HL
man_entity_tilerow_platform_update::
  .macro _mult_a_4
    add a
    add a
  .endm

  _elementos_restantes_plataforma = .+1
  ld a, #0x00

  ld a, #0x10
  ld (_elementos_restantes_plataforma), a

  _tilerow_platform_update_loop:
  push ix
  ld a, p_type(ix)

  ; Deteccion tipo pincho
  cp #64
  jr z, #_generar_plataforma_pincho

  cp #65
  jr z, #_generar_plataforma_pincho

  cp #66
  jr z, #_generar_plataforma_pincho

  cp #67
  jr z, #_generar_plataforma_pincho

  _continue_checking:
  
  ; Deteccion tipo plataforma
  and #0b11100000
  cp  #0b00100000
  jr z, #_generar_plataforma_normal

  _tilerow_platform_update_continue: 
  pop ix
  inc ix
  ld a, (_elementos_restantes_plataforma)
  dec a
  ret z         ; Comprobamos si hemos llegado al final del bucle

  ld (_elementos_restantes_plataforma), a
  jr #_tilerow_platform_update_loop

    _generar_plataforma_normal:
    ld hl, #_default_platform
    call man_platform_create
    jr #_continue_generation

    _generar_plataforma_pincho:
    ld hl, #_default_pincho
    call man_platform_create
    jr #_continue_generation


  _continue_generation:
  
  ld a, (_elementos_restantes_plataforma)
  ld b, a
  ld a, #16
  sub b
  _mult_a_4 ; ld a, #16 ; ld b, (elemntos), sub b

  ld p_x(ix), a

  ld a, #POSICION_INFERIOR_PLATAFORMA
  ld p_y(ix), a


  jr #_tilerow_platform_update_continue


  ret




; Mueve la posicion de la plataforma
; pasada por argumento 8 posiciones
; hacia arriba para mantener las 
; coordenadas relativas a la pantalla.
; Si la posicion traspasa 0x00 se marca
; como plataforma a destruir
; Input:
;   IX -> Dir. mem. de la plataforma.
; Modifica:
;   AF
man_platform_one_move_single_scroll::
  ld a, p_y(ix)
  sub #NUM_POSICIONES_SCROLL  

  cp #0x00                      ; No queremos borrarla en 0x00, la queremos borrar cuando lo transpasa
  jr z, #_one_move_single_scroll_NO_marcar_destruir ; Si se pasa la posicion 0x00

  ld p_y(ix), a   ; Actualizamos la posicione
  ret

  _one_move_single_scroll_NO_marcar_destruir:
    call man_platform_set4destruction     ; La marcamos para destruir

RET


; Mueve la posicion de todas las plataformas
; en el array de plataformas
; Modifica:
;   AF, BC, IX
man_platform_update_forall_position::
  call platformman_getPlatformArray_IX

  _update_forall_position_loop:
    ld a, p_type(ix)
    cp #E_TYPE_INVALID
    ret z ; Si es de tipo invalido, salimos del bucle

    ld a, p_y(ix)
    sub #NUM_POSICIONES_SCROLL
    jr nz, #_update_forall_position_NO_marcar_destruir

    call man_platform_set4destruction     ; La marcamos para destruir si (p_y - #0xF8 == 0)

    _update_forall_position_NO_marcar_destruir:
    ld p_y(ix), a
    ld bc, #sizeof_p
    add ix, bc      ; Iteramos al siguiente elemento
    jr #_update_forall_position_loop



RET

