;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; TILEMAP 
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Includes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "tilemap.h.s"
.include "platform.h.s"
.include "../sys/render.h.s"
.include "entity.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

previous_map_line_mem::           .dw #0x0000
current_map_line_mem::            .dw #0x0000
current_map_line_platforms_mem::  .dw #0x0000

current_tiles_mem::               .dw #0x0000


current_offset_y:                 .db #0x00
threshold_offset_incremented:     .db #0x00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Funciones
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; Setea la dir. de mem. de
; la linea del mapa actual.
; Input:
;   HL -> Dir. mem. linea
;         mapa.
sys_tilemap_set_current_map_line_mem::
  ld (current_map_line_platforms_mem), hl

  ld bc, #0x0110 ; Default 0x00
  add hl, bc
  
  ld (current_map_line_mem), hl
  ld (previous_map_line_mem), hl
  ret

; Setea la dir. de mem. del
; primer tile.
; Input:
;   HL -> Dir. mem. tile.
sys_tilemap_set_current_tile_mem::
  ld (current_tiles_mem), hl
  ret


; Actualiza el mapa que se muestra
; por pantalla dibujando A filas
; y realizando A scrolls.
; Input:
;   A -> Numero de filas que hay que dibujar 
; Modifica:
;   AF, HL, BC, DE, IX
sys_tilemap_update::

    ld ix, (current_map_line_platforms_mem)
    call man_entity_tilerow_platform_update     ; Generamos el row de plataformas

    ld ix, (previous_map_line_mem)
    call man_tilerow_entity_update              ; Generamos el row de enemigos
    
    ;call man_entity_tilerow_entity_update              ; Generamos las entidades
 

    call man_platform_update_forall_position    ; Actualizamos las posiciones de las
                                                ; plataformas tras mover la pantalla
                                                ; La posicion solo se utiliza para     
                                                ; colisiones, no para renderizado!!!   
                                                ;
                                                ; Seteamos las plataformas que pasan
                                                ; el limite superior de la pantalla
                                                ; para que se elminen
    call man_entity_update_forall_position

    ld ix, (current_map_line_mem)
    ld (previous_map_line_mem), ix  ; Guardamos la posicion previa para el renderizado

    ld iy, (current_map_line_platforms_mem)

    ld bc, #0x10
    add ix, bc
    add iy, bc

    ld (current_map_line_mem), ix    ; Actualizar a la siguiente posicion del mapa
    ld (current_map_line_platforms_mem), iy    ; Actualizar a la siguiente posicion del mapa

    ld a, #0x01
    call sys_render_set_apply_scroll

RET




; Incrementa en una posicion el offset
; restante del scroll
; Modifica:
;   HL
incrementar_offset_actual_y::
  ld hl, #current_offset_y
  inc (hl)
  ret


; Setea el threshold_offset_incremented
; Input:
;   A -> Valor a setear
; Modifica:
;   HL
set_tilemap_set_offset_incremented_y::
  ld hl, #threshold_offset_incremented
  ld (hl), a
  ret



; Genera las plataformas de una fila
; del tilemap posicinadas en la
; parte inferior de la pantalla.
; Input:
;   IX -> Fila de datos de entidades
; Modifica:
;   HL
man_tilerow_entity_update::
  .macro _mult_a_4
    add a
    add a
  .endm

  _elementos_restantes_entity = .+1
  ld a, #0x10

  _tilerow_entity_update_loop:
  push ix
  ld a, (ix)

  cp #TIPO_PATROL                  ; Si es entidad -> La creo
  jr z, #_generar_tipo_patrol

  cp #TIPO_BAT                  ; Si es entidad -> La creo
  jr z, #_generar_tipo_bat

  cp #TIPO_LASER                  ; Si es entidad -> La creo
  jr z, #_generar_tipo_laser

  _tilerow_entity_update_continue: 
  pop ix
  inc ix
  ld a, (_elementos_restantes_entity)
  dec a
  jr z, #_fin_reset_loop

  ld (_elementos_restantes_entity), a
  jr #_tilerow_entity_update_loop

  _generar_tipo_patrol::
  call create_turtle

  ld a, (_elementos_restantes_entity)
  ld b, a
  ld a, #16
  sub b
  _mult_a_4
  LD X_ENTITY_POS(IX), A
  ADD VY_MAX_POS(IX)
  LD VX_MAX_POS(IX), A

  jr #_continue_tilerow_entity_update

  _generar_tipo_bat:
  call create_bat
  jr #_continuar_tilerow_entity_update

  _generar_tipo_laser:
  call create_laser
  jr #_continuar_tilerow_entity_update


  _continuar_tilerow_entity_update:
  ld a, (_elementos_restantes_entity)
  ld b, a
  ld a, #16
  sub b
  _mult_a_4

  LD X_ENTITY_POS(IX), A

  jr #_continue_tilerow_entity_update
  

  _continue_tilerow_entity_update:
  ld a, (_elementos_restantes_entity)
  ld b, a
  ld a, #16
  sub b
  _mult_a_4

  ld X_ENTITY_POS(ix), a

  ld a, #0xE0
  ld Y_ENTITY_POS(ix), a


  jr #_tilerow_entity_update_continue


  _fin_reset_loop:
  ld hl, #_elementos_restantes_entity
  ld (hl), #0x10

  ret

