;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------
;
; Animations System 
;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Includes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "man/entity.h.s"
.include "cpcteleraHeader.h.s"
.include "animations.h.s"
.include "man/animations.h.s"
.include "sys/render.h.s"
.include "sys/collisions.h.s"
.include "cpctelera.h.s"
.include "input.h.s"

standing_right:
    ld a, (actual_action)
    and #action_left
    jr z, not_action_left_r
    
    ld a, (collision_flag)
    and #col_flag_up
    jr nz, not_col_up_r
        ld hl, #_animation_player_falling_left
        ld a, #anim_status_player_falling_left
        ld ANIM_STATUS(ix), a
        jr finish_looking_r
    not_col_up_r:
        ld hl, #_animation_player_standing_left
        ld a, #anim_status_player_standing_left
        ld ANIM_STATUS(ix), a
        jr finish_looking_r

    not_action_left_r:
        ld a, (collision_flag)
        and #col_flag_up
        ret nz
        
        ld hl, #_animation_player_falling
        ld a, #anim_status_player_falling
        ld ANIM_STATUS(ix), a
        jr finish_looking_r

    finish_looking_r:
        ld ANIMATION_POS_H(ix), h
        ld ANIMATION_POS_L(ix), l    
    ret

standing_left:
    ld a, (actual_action)
    and #action_right
    jr z, not_action_right_l
    
    ld a, (collision_flag)
    and #col_flag_up
    jr nz, not_col_up_l
        ld hl, #_animation_player_falling
        ld a, #anim_status_player_falling
        ld ANIM_STATUS(ix), a
        jr finish_looking_l
    not_col_up_l:
        ld hl, #_animation_player_standing_right
        ld a, #anim_status_player_standing_right
        ld ANIM_STATUS(ix), a
        jr finish_looking_l

    not_action_right_l:
        ld a, (collision_flag)
        and #col_flag_up
        ret nz
        
        ld hl, #_animation_player_falling_left
        ld a, #anim_status_player_falling_left
        ld ANIM_STATUS(ix), a
        jr finish_looking_l

    finish_looking_l:
        ld ANIMATION_POS_H(ix), h
        ld ANIMATION_POS_L(ix), l    
    ret

falling:
    ld a, (actual_action)
    and #action_left
    jr z, not_action_left_f
    ld hl, #_animation_player_falling_left
    ld a, #anim_status_player_falling_left
    ld ANIM_STATUS(ix), a
    jr finish_falling_r
    not_action_left_f:
        ld a, (collision_flag)
        cp #col_flag_up
        ret nz
            ld hl, #_animation_player_polvo
            ld a, #anim_status_player_standing_right
            ld ANIM_STATUS(ix), a

    finish_falling_r:
    ld ANIMATION_POS_H(ix), h
    ld ANIMATION_POS_L(ix), l    

    ret

falling_left:
    ld a, (actual_action)
    and #action_right
    jr z, not_action_right_f
    ld hl, #_animation_player_falling
    ld a, #anim_status_player_falling
    ld ANIM_STATUS(ix), a
    jr finish_falling_l

    not_action_right_f:
        ld a, (collision_flag)
        cp #col_flag_up
        ret nz
            ld hl, #_animation_player_polvo_left
            ld a, #anim_status_player_standing_left
            ld ANIM_STATUS(ix), a

    finish_falling_l:
    ld ANIMATION_POS_H(ix), h
    ld ANIMATION_POS_L(ix), l    
    
    ret

laser_on:
    ld a, VX_MAX_POS(ix)
    cp #1
    ret z
    ld hl, #_animation_laser_off
    ld a, #anim_status_laser_off
    ld ANIM_STATUS(ix), a
    ld ANIMATION_POS_H(ix), h
    ld ANIMATION_POS_L(ix), l    
    
    ret


laser_off:
    ld a, VX_MAX_POS(ix)
    cp #0
    ret z
    ld hl, #_animation_laser_on
    ld a, #anim_status_laser_on
    ld ANIM_STATUS(ix), a
    ld ANIMATION_POS_H(ix), h
    ld ANIMATION_POS_L(ix), l    
    
    ret

viser_detected:
    ld hl, #_animation_viser_detected
    ld a, #0
    ld ANIM_STATUS(ix), a
    ld ANIMATION_POS_H(ix), h
    ld ANIMATION_POS_L(ix), l    
    ret

check_change_animation::
    ld a, ANIM_STATUS(ix)
    cp #anim_status_laser_on
    jr nz, not_laser_on
        call laser_on
        ret
    not_laser_on:
    cp #anim_status_laser_off
    jr nz, not_laser_off
        call laser_off
        ret
    not_laser_off:
    cp #anim_status_player_standing_left
    jr nz, not_standing_left
        call standing_left
        ret
    not_standing_left:
    cp #anim_status_player_standing_right
    jr nz, not_standing_right
        call standing_right
        ret
    not_standing_right:
    cp #anim_status_player_falling
    jr nz, not_falling
        call falling
        ret
    not_falling:
    cp #anim_status_player_falling_left
    jr nz, not_falling_left
        call falling_left
        ret
    not_falling_left:
    cp #anim_status_viser_detected
    jr nz, not_viser_detected
        call viser_detected
        ret
    not_viser_detected:
    ret
update_animation::
    ld e, SPRT_ENTITY_POS_L(ix)
    ld d, SPRT_ENTITY_POS_H(ix)
    ld PREV_SPRT_ENTITY_POS_L(ix), e
    ld PREV_SPRT_ENTITY_POS_H(ix), d

    ld__hl_ix
    ld e, ANIMATION_POS_L(ix)
    ld d, ANIMATION_POS_H(ix)

    ex de, hl  
    _continueAnim:
    ld c, (hl)
    inc hl
    ld b, (hl)
    inc hl

    ld a, c
    or b
    jr z, _resetAnim

    ld ANIMATION_POS_L(ix), l
    ld ANIMATION_POS_H(ix), h


    ld SPRT_ENTITY_POS_L(ix), c
    ld SPRT_ENTITY_POS_H(ix), b

    ret
    _resetAnim:
        ld a, (hl)
        inc hl
        ld h, (hl)
        ld l, a
        jr _continueAnim
sys_animations_update_one_entity::
    call check_change_animation
    call update_animation
    ret
    
    ;ld SPRT_ENTITY_POS_L(ix), #<_spr_enemy1_1
    ;ld SPRT_ENTITY_POS_H(ix), #>_spr_enemy1_1


sys_animations_update::
    ld hl, #sys_animations_update_one_entity
    ld a, #E_TYPE_ANIMATED                            ;TODO: Not enough flags
    call man_entity_getArray
    call man_entity_forall_matching
    ret
