;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------
;
; Input System
;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Includes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cpcteleraHeader.h.s"
.include "collisions.h.s"
.include "input.h.s"
.include "man/entity.h.s"
.include "physics.h.s"
.include "man/music.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Array of keys, actions
;;
key_actions::
    .dw Key_Q,      action_move_up
    .dw Joy0_Up,    action_move_up
    .dw Key_O,      action_move_left
    .dw Joy0_Left,  action_move_left
    .dw Key_P,      action_move_right
    .dw Joy0_Right, action_move_right
    .dw Key_R,      action_restart_game
    .dw Key_M,      action_toggle_music
    .dw 0

looking_direction:: .db DIR_RIGHT

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Array of keys, actions
;;
actual_action::
    .db 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sets the y velocity to 0
;;  INPUT:  
;;      IX: Pointer to entity
;;  DESTROYS: -
dont_move_y:
    ld VY_ENTITY_POS(ix), #0 
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sets the x velocity to 0
;;  INPUT:  
;;      IX: Pointer to entity
;;  DESTROYS: -
dont_move_x:
    ld VX_ENTITY_POS(ix), #0
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sets the y velocity to -2 to move up if the player is not 
;; collisioning wirh the bottom part of the platform
;;  INPUT:  
;;      IX: Pointer to entity
;;  DESTROYS: AF, IY
action_move_up::
    ld a, (player_in_air)
    cp #0
    RET  NZ

    ld a, (actual_action)
    or #action_up
    ld (actual_action), a

        LD   A, #JUMP_SFX
        ;CALL man_music_activate_sfx

        LD  A, #1
        ld (player_in_air), a                  ; Set flag to in the air, it cannot jump again

        LD C, #0
        LD B, #JUMP_FORCE
        CALL sys_physics_apply_force
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sets the x velocity to 1 to move right if the player is not 
;; collisioning with the left part of the platform
;;  INPUT:  
;;      IX: Pointer to entity
;;  DESTROYS: AF, IY
action_move_right:
    ld a, (actual_action)
    or #action_right
    ld (actual_action), a

    LD   C, #PLAYER_X_FORCE                  ; Movement on X axis
    LD   B, #0                               ; Don't move on Y axis
    CALL sys_physics_apply_force
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sets the X velocity to -1 to move left if the player is not 
;; collisioning with the right part of the platform
;;  INPUT:  
;;      IX: Pointer to entity
;;  DESTROYS: AF, IY
action_move_left:
    ld a, (actual_action)
    or #action_left
    ld (actual_action), a

    LD   C, #-PLAYER_X_FORCE                 ; Movement on X axis
    LD   B, #0                               ; Don't move on Y axis
    CALL sys_physics_apply_force    
    ret


;;Devuelve action_restart en la flag actual_action
action_restart_game::
    ld a, #action_restart
    ld (actual_action), a

    ret

;; Alterna el estado de la musica, sonando o no
action_toggle_music:
    CALL man_music_toggle
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Check keyboard keys
;;  INPUT:  
;;      IX: Pointer to entity
;;  DESTROYS: AF, BC, DE, HL, IX, IY
sys_input_check_keyboard:
    ld VX_ENTITY_POS(ix), #0        ; Sets the velocities to 0 TODO
    LD   IY, #key_actions-4         ; Sets iy to the position of the key actions - 4 to allow to sum every loop
    LD   (ptr_iy_input), IY         ; Save the iy value with automodified code
loop_keys:
    ptr_iy_input = .+2
    LD   IY, #0
    LD   BC, #4                 ;/ IY += 4 (Next key)
    ADD  IY, BC                 ;\

    LD   L, key_l(IY)           ;/ HL = Next key
    LD   H, key_h(IY)           ;\

    ;; Check if key==null
    LD   A, L                   ;/ A = H | L
    OR   H                      ;\
    RET  Z                      ; if A==0, Key==null, ret

    ; Check if key is pressed
    CALL cpct_isKeyPressed_asm
    LD   (ptr_iy_input), IY
    JR   Z, loop_keys

    ; Key is pressed, execute action
    LD   HL, #loop_keys         ; ret to Loop_keys
    PUSH HL
    LD   L, action_l(IY)        ;/ HL = action function pointer
    LD   H, action_h(IY)        ;\
    LD   (ptr_iy_input), IY
    JP   (HL)                   ;;Jump to action function

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sets the y velocity to 2 to move down if the player is not 
;; collisioning wirh the upper part of the platform
;;  INPUT:  -
;;  DESTROYS: AF, IX, IY, DE, BC
sys_input_update::
    ld a, #0
    ld (actual_action), a
    ld hl, #sys_input_check_keyboard
    ld a, #E_TYPE_PLAYER
    call man_entity_forall_matching
    ret


