;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; RENDER 
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Includes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.include "render.h.s"
.include "man/entity.h.s"
.include "man/platform.h.s"
.include "cpctelera.h.s"
.include "cpcteleraHeader.h.s"
.include "man/tilemap.h.s"
.include "man/level.h.s"

.include "ai.h.s"

.globl _tiles00_000

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
scroll_base: .dw #0x0030
scroll_offset: .dw #0x0000


curr_map_line:  .dw #0x0000
curr_tiles:     .dw #0x0000


vid_mem_next_platform: .dw #0xC780
apply_scroll:  .db #0x00  ; 0 -> No
                          ; 1 -> Si

start_video_dir: .dw #0xC000

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Funciones
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Setea aplicar scroll
; Input:
;   A -> Nuevo valor
sys_render_set_apply_scroll::
  ld (apply_scroll), a
  ret

; Obtiene la direccion del primer byte
; de la memoria de video.
; RelativeMem = #C000 + 2*(scroll_offset)
; Output:
;   DE -> Inicio mem. vid. actual.
; Modifica:
;   HL, DE
sys_render_getStartVideoDir::
  ld de, (start_video_dir)
RET

; Setea la dir. de mem. de
; la linea del mapa actual.
; Input:
;   HL -> Dir. mem. linea
;         mapa.
sys_render_setCurrMapLine::
  ld (curr_map_line), hl
  ret


; Setea la dir. de mem. del
; primer tile.
; Input:
;   HL -> Dir. mem. tile.
sys_render_setCurrTiles::
  ld (curr_tiles), hl
  ret




; Obtiene la direccion del primer byte
; de la memoria de video.
; Output:
;   HL -> Offset
; Modifica:
;   HL
sys_render_getOffset_HL::
  ld hl, (scroll_offset)
  ret

; Setup amstrad. Establece 26 chars horizontales
; y 35 chars verticales de pantalla visible.
; Alinea la pantalla horizontal y verticalmente.
; Establece el borde negro.
; Modifica:
;   AF, BC, HL
sys_render_displaySetUp::
  ; halt
  ld c, #0
  call cpct_setVideoMode_asm

  cpctm_clearScreen_asm #0x00 ; Limpiamos la pantalla

  ; Register 1 -> Hor. disp = 32 chars = 64 bytes modo 0
  ld bc, #0xbc01
  out (c), c
  ld bc, #0xbd20
  out (c), c

  ; Register 2 -> Hor. sync = 28 -> Alineado centro
  ld bc, #0xbc02
  out (c), c
  ld bc, #0xbd2a
  out (c), c

  ; Register 6 -> Ver. disp = 32 chars = 64 bytes
  ld bc, #0xbc06
  out (c), c
  ld bc, #0xbd1e
  out (c), c

  ; Register 7 -> Ver. sync = 30 chars -> Alineado arriba
  ld bc, #0xbc07
  out (c), c
  ld bc, #0xbd21
  out (c), c

  LD   HL, #_g_palette
  LD   DE, #16
  CALL cpct_setPalette_asm

  cpctm_setBorder_asm HW_BLACK

  ld hl, #0xC000
  ld (start_video_dir), hl

  ld hl, #0x0000
  ld (scroll_offset), hl

  ld hl, #0xC780
  ld (vid_mem_next_platform), hl
  ret


; Dibuja un rectangulo por pantalla. En modo 0
; el ancho son 2 pixeles.
;
; Input:
;   DE -> Puntero a la direccion de memoria 
;         de la esquina superior izquierda 
;   A -> Color del rectangulo
;   C -> Ancho del rectangulo
;   B -> Alto del rectangulo
; Modifica:
;   AF, BC, DE, HL
sys_render_drawSolidBox::

  .macro COPY2HL_n_INC reg8
     ld   (hl), reg8         ;; [2] (HL) = reg8
     inc   hl                ;; [2] HL++  
  ld bc, #0xbca2
  out (c), c
  ld bc, #0xbd0C
  out (c), c

  .endm
     
     ld     a, #128          
     sub    c                
     sub    c                ;; [1]
     ld (_jr_offset), a      
     
     ld     c, l             
     ld     h, d             ;; [1] / HL = DE
     ld     l, e             

  _next_line:
  _jr_offset = .+1
     jr__0                   
                             
  .rept 64
     COPY2HL_n_INC c         
  .endm
   
     dec    b                
     ret    z                
     
     ld     a, #0x8          
     add    d                
     ld     h, a             
     ld     d, a             
     ld     l, e             
     
     and   #0x38             
     jp    nz, _next_line    
                             
     
     ld    hl, #0xC040
     add   hl, de            
     ld     d, h             ;; [1] / DE = HL
     ld     e, l             ;; [1] \
     jp   _next_line         



; Dibuja un sprite en la direccion
; de memoria de video especificada.
; Input:
;   HL -> Puntero al sprite
;   DE -> Puntero a la memoria de video
;   C -> Ancho del sprite
;   B -> Alto del sprite
; Modifica:
;   AF, BC, DE, HL
sys_render_drawSprite::

     ld    a, #126           
     sub   c                 
     sub   c                 
     ld (ds_drawSpriteWidth+#4), a 

     ld    a, b              
     ex   de, hl             
                             

  ds_drawSpriteWidth_next:
     ex   de, hl             
                             
  ds_drawSpriteWidth:
     ld   bc, #0x800  
                      
     jr__0            
                      
     ldi              
     ldi              
     ldi              
     ldi              
     ldi              
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
     ldi              ;; [5] <|
     ldi              ;; [5] <|
     ldi              ;; [5]  |
     ldi              ;; [5]  |
   
     dec   a          
     ret   z          

     
     ex   de, hl      
     add  hl, bc      
     ld    b, a       
     ld    a, h       
                      
     and   #0x38      
     ld    a, b       
     jp   nz, ds_drawSpriteWidth_next 
                                      

     ld   bc, #0xC040           
     add  hl, bc                
     jp ds_drawSpriteWidth_next 



; Actualiza el offset para bajar la pantalla
; 8 bytes.
; Modifica:
;   BC, HL, AF
scroll_down::
  ld hl, (scroll_offset)
  ld bc, #0x20
  add hl, bc



  ld a, h
  and #3
  cp #0x00
  jr nz, #_continue_scroll_down
  ld h, a
  ld a, l
  cp #0x00
  jr nz, #_continue_scroll_down_2

  push hl
  ld hl, #man_entity_increment_prev_pos_y
  ld a, #E_TYPE_RENDER
  call man_entity_forall_matching
  pop hl

  _continue_scroll_down:
  ld h,a

  _continue_scroll_down_2:
  ld (scroll_offset), hl

  push hl
  ld de, #0xC000
  add hl, hl
  add hl , de
  ld (start_video_dir), hl
  pop hl


  ld bc,#0xbc0c				;; select CRTC register 12
  out (c), c
  ld b, #0xbd				;; B = I/O address for CRTC register write

  ;; combine with scroll base
  ld a, (scroll_base)
  or h
  out (c),a

  ld bc, #0xbc0d				;; select CRTC register 13
  out (c),c
  ld b, #0xbd				;; B = I/O address for CRTC register write
  out (c),l




  ret


; HL
; Actualiza la posicion de memria inicial
; en el registro de CRTC.
; Input:
;   HL -> offset
; Modifica:
;   BC, HL, AF
set_offset::
  ld bc,#0xbc0c				;; select CRTC register 12
  out (c), c
  ld b, #0xbd				;; B = I/O address for CRTC register write

  ;; combine with scroll base
  ld a, (scroll_base)
  or h
  out (c),a

  ld bc, #0xbc0d				;; select CRTC register 13
  out (c),c
  ld b, #0xbd				;; B = I/O address for CRTC register write
  out (c),l
  ret


  


; Obtiene la direccion de la memoria de video
; dada una posicion. 
; Input:
;   DE -> Inicio dir. mem. video
;   C -> Columna. [0-64]
;   B -> Fila. [0-237]
; Output:
;   HL
; Modifica:
;   BC, HL, AF, DE
sys_render_getScreenPtr::
   ld    a, b     
   and   #0x07    
   ld    h, a     ;; [1] \ rH = Y % 8      ;; << rH contains Line number [0-7] inside character Row

   xor   b        

   ld l, a ; l = 8*R

   add   hl, hl   ;; hl + hl = 2hl
   add   hl, hl   ;; 2*(2hl)
   add   hl, hl   ;; 2*(4hl)
   ; 8 * (8R + 256L)

   ld     b, #0   ;; 
   add   hl, bc   ;; 
   add   hl, de   ;; 

   ld a, h
   cp #0xC0
   ret nc

   ld a, #0xF0
   ld h, a

   ld a, #0xC0
   add e
   ld e, a

   ret            ;; ret 8 * (8*R + 256*L) + x

sys_render_first_one_entity::

    ld a, TYPE_ENTITY_POS(ix)
    and #e_type_dead
    ret nz

    ld RENDER_FUNCTION_POS_L(ix), #<sys_render_one_entity
    ld RENDER_FUNCTION_POS_H(ix), #>sys_render_one_entity

    call sys_render_getStartVideoDir
    ld c, X_ENTITY_POS(ix)
    ld b, Y_ENTITY_POS(ix)
    call sys_render_getScreenPtr

    ex de, hl
    ld h, SPRT_ENTITY_POS_H(ix)
    ld l, SPRT_ENTITY_POS_L(ix)
    ld c, H_ENTITY_POS(ix)
    ld b, W_ENTITY_POS(ix)
    call sys_render_drawSpriteBlended

RET

sys_render_one_entity::

    call sys_render_getStartVideoDir
    ld c, PREV_PTR_POS_L(ix)
    ld b, PREV_PTR_POS_H(ix)
    call sys_render_getScreenPtr

    ex de, hl
    ld h, PREV_SPRT_ENTITY_POS_H(ix) 
    ld l, PREV_SPRT_ENTITY_POS_L(ix)
    ld c, H_ENTITY_POS(ix)
    ld b, W_ENTITY_POS(ix)
    call sys_render_drawSpriteBlended ; Borrado de la entidad en la prev. pos.

    ld a, TYPE_ENTITY_POS(ix)
    and #e_type_dead
    ret nz

    call sys_render_getStartVideoDir
    ld c, X_ENTITY_POS(ix)
    ld b, Y_ENTITY_POS(ix)
    call sys_render_getScreenPtr

    ex de, hl
    ld h, SPRT_ENTITY_POS_H(ix)
    ld l, SPRT_ENTITY_POS_L(ix)
    ld c, H_ENTITY_POS(ix)
    ld b, W_ENTITY_POS(ix)
    call sys_render_drawSpriteBlended ; Pintado de la entidad en la nueva pos.

RET        


sys_render_update::
    ld a, (apply_scroll)
    cp #0x00

    jr z, #_render_update_not_scroll

    ld de, (vid_mem_next_platform)
    ld hl, (current_tiles_mem)
    ld ix, (previous_map_line_mem)
    call sys_render_drawTileRow_4x8  ; Dibujamos una fila

    call sys_render_update_vid_mem_next_platform

    call man_level_dec_chunks_restantes

    call scroll_down

    _render_update_not_scroll:
    call man_entity_forall_render


    ld a, #0x00
    ld (apply_scroll), a
RET



; Pintando por XOR. La operacion es
; autosemejante, es decir, si se llama
; una vez pinta el sprite, y si se 
; vuelve a llamar lo borra.
; Input:
;   DE -> Dir. mem. video
;   HL -> Dir. sprite
;   C -> Altura del sprite
;   B -> Ancho del sprite
; Modifica:
;   AF, HL, DE, BC
sys_render_drawSpriteBlended::
   push ix         ;; [5] Save IX regiter before using it as temporal var
   ld__ixl_b       ;; [3] Save Sprite Width into IXL for later use

     ;; Copy and blend all the rows of the sprite
  height_loop:
     push de         ;; [4] Save DE for later use (jump to next screen line)

     ;; Copy and blend 1 complete sprite row
  width_loop:
     ld     a,(de)   ;; [2] Get next background byte into A

    ;; Blending Function. This is a 1-byte Z80 operation that should blend
    ;; the byte in A (background) with the byte from the sprite at (HL)
    ;; This function is modified by <cpct_setBlendMode>.
  _cpct_dsb_blendFunction::
     xor (hl)        ;; [2] Blend background with Sprite || This byte will be modified to change blend function

     ld  (de), a     ;; [2] Save background blended with sprite
     inc   de        ;; [2] | Point to next byte 
     inc   hl        ;; [2] |  (sprite and memory)

     djnz  width_loop;; [3/4] B holds sprite width. Decrease it and loop until B=0

     pop  de         ;; [3] Recover DE from stack. We use it to calculate start of next pixel line on screen

     dec   c         ;; [1] C holds sprite height. We decrease it to count another pixel line finished
     jr    z,copy_ended;; [2/3] If 0, we have finished the last sprite line.
                                  ;;      - If not 0, we have to move pointers to the next pixel line

     ld__b_ixl       ;; [3] Restore Sprite Width into B

     push hl
     ld hl, #0x0040
     add hl, de
     pop hl
     jp c, _limit_crossed

     ld    a, d      ;; [1] Start of next pixel line normally is 0x0800 bytes away.
     add   #0x08     ;; [2]    so we add it to DE (just by adding 0x08 to D)
     ld    d, a      ;; [1]
     and   #0x38     ;; [2] We check if we have crossed memory boundary (every 8 pixel lines)
     jr   nz,height_loop  ;; [2/3]  by checking the 4 bits that identify present memory line. (......11 1.......)
                          ;; ....  If 0, we have crossed boundaries

  _8bit_boundary_crossed:
     ld    a, e      ;; [1] de = de + 0xc050h
     add   #0x40     ;; [2] -- relocate de pointer to the start of the next pixel line:
     ld    e, a      ;; [1] -- de is moved forward 3 memory banks plus 50 bytes (4000h * 3) 
     ld    a, d      ;; [1] -- which effectively is the same as moving it 1 bank backwards and then
     adc   #0xc0     ;; [2] -- 50 bytes forwards (which is what we want to move it to the next pixel line)
     ld    d, a      ;; [1] -- calculations are made with 8 bit maths as it is faster than other alternatives here

     jr  height_loop ;; [3] Jump to continue with next pixel line

  copy_ended:
     pop  ix         ;; [5] Restore IX before returning
     ret             ;; [3] Return to caller


_limit_crossed:
   ld    a, e      ;; [1] de = de + 0xc050h
   add   #0x40     ;; [2] -- relocate de pointer to the start of the next pixel line:
   ld    e, a      ;; [1] -- de is moved forward 3 memory banks plus 50 bytes (4000h * 3) 
   ld    d, #0xC0
   jr height_loop




;(1B C) width	Width in tiles of the view window to be drawn
;(1B B) height	Height in tiles of the view window to be drawn
;(2B DE) tilemapWidth	Width in tiles of the complete tilemap
;(2B HL) tileset	Pointer to the start of the tileset definition (list of 32-byte tiles).
sys_render_tilemap::

  ret

sys_render_drawTilemap4x8_ag_asm::
  ret


; Dibuja un unico tile en pantalla.
; Asume que la dir. de mem. de video
; se encuentra en una sola fila 
; (8 bytes).
; Input:
;   HL -> Dir. mem. tile
;   DE -> Dir. mem. video (destino)
; Modifica:
;   AF, BC, DE, HL
sys_render_drawTile_4x8::

  ld a, #0x08

  _drawTile_Row:
  push af
  push de

  ld bc, #0x0004
  ldir            ; Pinta una fila de bytes

  pop de

  pop af
  dec a
  ret z           ; Terminamos de dibujar el tile

  push hl         
  ex de, hl
  ld de, #0x0800
  add hl, de
  ex de, hl       ; +800 -> Siguiente linea


  pop hl

  jr #_drawTile_Row


; Dibuja una fila de 16 tiles.
; Input:
;   HL -> Dir. mem. tiles
;   DE -> Dir. mem. video (destino)
;   IX -> Dir. mem. mapa
; Modifica:
;   AF, BC, DE, HL
sys_render_drawTileRow_4x8::
  .macro _mult_32_hl
    add hl, hl
    add hl, hl
    add hl, hl
    add hl, hl
    add hl, hl
  .endm

  ld a, #16

  _draw_tile_row:
    push af
    push hl
    push de

    ld b, h
    ld c, l

    ld h, #00
    ld l, (ix)  ; Cargamos el valor del primer elemento del mapa
    _mult_32_hl ; Los tiles estan separados 32 bytes.
    add hl, bc

    ;add hl, ix

;   HL -> Dir. mem. tile
;   DE -> Dir. mem. video (destino)
    call sys_render_drawTile_4x8
    
    pop de

    ld hl, #0x0004
    add hl, de
    ex de, hl   ; Nos movemos a la siguiente posicion de video

    pop hl
    pop af

    dec a
    ret z

    inc ix      ; Nos movemos al siguiente tile del mapa

    jr #_draw_tile_row

; Actualiza el valor vid_mem_next_platform
; Destroy:
;   HL, BC
sys_render_update_vid_mem_next_platform::
  ld hl, (vid_mem_next_platform)
  ld bc, #0x0040
  add hl, bc
  ld (vid_mem_next_platform), hl

  ld a, h
  cp #0xC8
  jr z, _vid_mem_next_limit_crossed
  RET

  _vid_mem_next_limit_crossed:
  ld h, #0xC0
  ld (vid_mem_next_platform), hl
RET




sys_render_menu::
RET
