;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of ziloggames_z80 : An Amstrad CPC Game on Z80
;;  Copyright (C) 2022 Ander (@ander-db) / Carlos (@carlosky66) / Adrian (@RosiYo)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------


.include "cpctelera.h.s"
.include "utils.h.s"
.include "man/entity.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Limits a value by E without changing sign
;; INPUT:
;;      A = Initial Value
;;      E = Limit value
;; DESTROYS: AF, DE
limit_value:
    CP #0xF0
    JR C, _positive_limit
        LD  D, A
        XOR A
        SUB D

        SUB E
        JR  C, under_negative_limit
            XOR A
            SUB E
            JR  _exit_limit
        under_negative_limit:
            LD  A, D
        JR  _exit_limit
    _positive_limit:
        CP E
        JR C, _exit_limit
            LD A, E
    _exit_limit:
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Apply
;; INPUT:
;;      A = Initial velocity in an axis
;;      E = Bit mask, number of bytes shifted
;; DESTROYS: AF, DE
shift_value:
    apply_shift:                ; Move E bits to the right
        SRA A
        DEC E
    JR  NZ, apply_shift
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Mult
;; INPUT:
;;      C = Num to apply
;;      B = Times
;; DESTROYS: AF, BC
mult_value:
    LD  A, B
    CP  #0
    RET Z
    
    XOR A

    mult_loop:
        DEC B
        RET Z
        ADD C
    JR  mult_loop
